
# TensorQuant

![TensorQuant Logo](https://img.shields.io/badge/TensorQuant-v0.0.3-blue.svg)
![Python](https://img.shields.io/badge/python-v3.10+-blue.svg)
![Build Status](https://img.shields.io/badge/build-passing-brightgreen.svg)
![License](https://img.shields.io/badge/license-MIT-green.svg)

**TensorQuant** is a Python financial library designed to provide a practical, Python-based implementations. Leveraging Tensor arrays, TensorQuant supports pricing, intensive risk management computations, and algorithmic differentiation. You can explore examples and use cases in the [**playground repository**](https://github.com/andrea220/tqPlayground) with Jupyter notebooks. For detailed API references and comprehensive documentation, visit the [**ReadTheDocs**](https://tquant.readthedocs.io/en/latest/index.html) page. 

It is particularly valuable in academic settings, such as the [Finance Master courses at the University of Siena](https://finance.unisi.it/it), where students gain hands-on experience with financial libraries and object-oriented programming.

Many of TensorQuant's components draw inspiration from the renowned [QuantLib](https://www.quantlib.org) library. Our thanks go to the QuantLib community for their contributions to financial modeling. While simplified for ease of use, TensorQuant aims to strike a balance between ease of understanding and professional architecture.

---

## 📑 Table of Contents

- [🌟 Features](#-features)
- [🛠️ Installation](#%EF%B8%8F-installation)
- [🚀 Usage](#-usage)
- [📝 License](#-license)
- [📧 Contact](#-contact)

---

## 🌟 Features

- **Tensor Array Operations**: Efficient handling and manipulation of tensor arrays for financial data.
- **Derivative Pricing**: Pricing financial derivatives.
- **Algorithmic Differentiation**: Automatic differentiation for optimization and sensitivity analysis.
- **Stochastic Models**: Simulations and solver tools for financial modeling.
- **Extensibility**: Easy to extend and customize for a wide range of financial applications.

---

## 🛠️ Installation

To install `TensorQuant`, use pip:

```bash
pip install tensorquant
```

Alternatively, clone the repository and install manually:

```bash
git clone https://github.com/andrea220/tQuant.git
cd tQuant
pip install .
```

---

## 🚀 Usage

To get started using `TensorQuant`, here are some resources:

### Examples
- Visit the [**`Playground`**](https://github.com/andrea220/tqPlayground) for Jupyter notebooks containing examples and use cases.

### Documentation
- The [**`ReadTheDocs`**](https://tquant.readthedocs.io/en/latest/index.html) page provides API references and comprehensive documentation.

### GitHub Repository
- Check out the open-source code on [**GitHub**](https://github.com/andrea220/tQuant).

---

## 📝 License

`TensorQuant` is licensed under the GPL-3.0 License. See the [LICENSE](LICENSE) file for more information.

---

## 📧 Contact

For any questions or suggestions, feel free to reach out:

- **Email**: [carapelliandrea@gmail.com](mailto:carapelliandrea@gmail.com)

---

Happy computing!
